﻿Event.observe(document, "dom:loaded", function () {
    scAttachEvents();
    var sortList = $("sort-list");
    if (sortList) {
        Sortable.create("sort-list", {
            scroll: "MainContainer", elements: $$(".sort-item.editable"), format: /^([0-9a-zA-Z]{32})$/, onUpdate: function () {
                scPersistSortOrder();
            }
        });
    }
});

function scAttachEvents() {
    $$(".sort-item.editable").each(function (item) {
        item.on("click", onItemClick);
        if (scForm.browser.isIE) {
            item.on("mouseenter", function () { this.addClassName("hover"); });
            item.on("mouseleave", function () { this.removeClassName("hover"); });
        }
    });

    Event.observe(document.body, "click", function () {
        scClearSelectedItem();
        scUpdateMoveButtonsState();
    });
};

function onItemClick(e) {
    e.stop();
    scClearSelectedItem();
    this.addClassName("selected");
    scUpdateMoveButtonsState();
};

function scMoveFirst() {
    var selectedItem = scGetSelectedItem();
    if (!selectedItem) {
        return false;
    }

    var first = scGetFirstSibling(selectedItem);
    if (!first) {
        return false;
    }

    first.insert({ before: selectedItem });
    scPersistSortOrder();
    scUpdateMoveButtonsState();
    return false;
};

function scMoveLast() {
    var selectedItem = scGetSelectedItem();
    if (!selectedItem) {
        return false;
    }

    var last = scGetLastSibling(selectedItem);
    if (!last) {
        return false;
    }

    last.insert({ after: selectedItem });
    scPersistSortOrder();
    scUpdateMoveButtonsState();
    return false;
};

function scMoveUp() {
    var selectedItem = scGetSelectedItem();
    if (!selectedItem) {
        return false;
    }

    var previous = scGetPreviousItem(selectedItem);
    if (!previous) {
        return false;
    }

    previous.insert({ before: selectedItem });
    scPersistSortOrder();
    scUpdateMoveButtonsState();
    return false;
};

function scClearSelectedItem() {
    var selectedItem = scGetSelectedItem();
    if (selectedItem) {
        selectedItem.removeClassName("selected");
    };
};

function scMoveDown() {
    var selectedItem = scGetSelectedItem();
    if (!selectedItem) {
        return false;
    }

    var next = scGetNextItem(selectedItem);
    if (!next) {
        return false;
    }

    next.insert({ after: selectedItem });
    scUpdateMoveButtonsState();
    scPersistSortOrder();
    return false;
};

function scDelete() {
    var deletedItems = $("deletedItems");
    if (!deletedItems) {
        deletedItems = new Element("input", { type: "hidden", id: "deletedItems" });
        var form = document.forms[0];
        if (!form) {
            return;
        }
        form.appendChild(deletedItems);
    }
    var selectedItem = scGetSelectedItem();
    selectedItem.addClassName("deleted");
    selectedItem.removeClassName("selected");
    var ids = $$(".deleted").map(function (item) { return item.id; }) || [];
    var serialized = ids.join("|") || "";
    deletedItems.value = serialized;
    scUpdateMoveButtonsState();
    return false;
};

function scGetSelectedItem() {
    return $$(".sort-item.selected")[0];
};

function scGetPreviousItem(item) {
    return item.previous(".sort-item:not(.deleted)");
};

function scGetNextItem(item) {
    return item.next(".sort-item:not(.deleted)");
};

function scGetFirstSibling(item) {
    return item.adjacent(".sort-item:not(.deleted)").first();
};

function scGetLastSibling(item) {
    return item.adjacent(".sort-item:not(.deleted)").last();
};

function scUpdateMoveButtonsState() {
    var moveUp = $("MoveUp");
    var moveDown = $("MoveDown");
    var deleteButton = $("Delete");
    var moveFirst = $("First");
    var moveLast = $("Last");
    var selectedItem = scGetSelectedItem();
    if (!selectedItem) {
        moveUp.disable();
        moveDown.disable();
        deleteButton.disable();
        moveFirst.disable();
        moveLast.disable();
        return;
    }
    deleteButton.enable();
    var next = scGetNextItem(selectedItem);
    if (next) {
        moveDown.enable();
        moveLast.enable();
    }
    else {
        moveDown.disable();
        moveLast.disable();
    }

    var previous = scGetPreviousItem(selectedItem);
    if (previous) {
        moveUp.enable();
        moveFirst.enable();
    }
    else {
        moveUp.disable();
        moveFirst.disable();
    }
};

function scPersistSortOrder() {
    var sortOrder = $("sortorder");
    if (!sortOrder) {
        sortOrder = new Element("input", { type: "hidden", id: "sortOrder" });
        var form = document.forms[0];
        if (!form) {
            return;
        }
        form.appendChild(sortOrder);
    }
    var ids = $$(".sort-item").map(function (item) { return item.id; }) || [];
    var serialized = ids.join("|") || "";
    sortOrder.value = serialized;
    scUpdateMoveButtonsState();
};
